﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using vJoyInterfaceWrap;
using System.IO.Ports;
using System.Timers;

namespace DDR_ArduinoFeeder
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	/// 

	public partial class MainWindow : Window
	{
		static vJoy controller = new vJoy();
		static uint id = 1;
		static bool skipRelease = false;
		static Timer buttonTimer = new Timer(1);
		static SerialPort port = new SerialPort();
		static int numberOfButtons = 4; //Should be 4 for UDLR arrows

		public MainWindow()
		{
			InitializeComponent();

			//vJoy initalization

			if (!controller.vJoyEnabled()) {
				System.Windows.MessageBox.Show("vJoy not detected. Make sure it's installed and running.");
				this.Close();
			}

			VjdStat status;

			status = controller.GetVJDStatus(id);

			if ((status == VjdStat.VJD_STAT_OWN) ||
				((status == VjdStat.VJD_STAT_FREE) && (!controller.AcquireVJD(id)))) {
				string prt = String.Format("Failed to acquire vJoy device number {0}.", id);
				System.Windows.MessageBox.Show(prt);
				this.Close();
			}

			SerialBox.Text = Properties.Settings.Default.COM_Port;
			ArduinoInit();

			ReleaseAll();
		}

		private void ArduinoInit()
		{
			port.Close();

			port.BaudRate = 9600;
			string ArduinoPort = Properties.Settings.Default.COM_Port;

			if (ArduinoPort == "") {
				return;
			}

			port.PortName = ArduinoPort;
			port.DtrEnable = true;
			port.RtsEnable = true;

			try {
				port.Open();
			} catch {
				string prt = String.Format("Failed to open port {0}. Make sure the Arduino is plugged in and working properly.", ArduinoPort);
				System.Windows.MessageBox.Show(prt);
				UIString.Text = "Feeder inactive";
				return;
			}

			buttonTimer.Elapsed += buttonEvent;
			buttonTimer.Enabled = true;

			UIString.Text = "Feeder active\non port " + ArduinoPort;
		}

		private void PushButton(uint button)
		{
			controller.SetBtn(true, id, button);
		}

		private void ReleaseButton(uint button)
		{
			controller.SetBtn(false, id, button);
		}

		private void ReleaseAll()
		{
			if (skipRelease) {
				skipRelease = false;
			} else {
				for (uint i = 1; i <= 4; i++) {
					controller.SetBtn(false, id, i);
				}
			}
		}

		private void buttonEvent(Object source, ElapsedEventArgs e)
		{
			try {
				string buffer = port.ReadExisting();
				if (buffer.LastIndexOf("\n") > 0) {
					int startPos = buffer.LastIndexOf("\n", buffer.LastIndexOf("\n") - 1);
					int length = buffer.LastIndexOf("\n") - startPos;
					string lastValues = buffer.Substring(startPos, length);
					lastValues = lastValues.Replace("\n", "");

					//Debug values; should be {off,on,on,off,on} with numberOfButtons at 5
					//lastValues = "0;21;30;5;60";

					for (uint i = 1; i <= numberOfButtons; i++) {
						int endPos = lastValues.IndexOf(";");
						int cutPos = lastValues.IndexOf(";") + 1;
						if (endPos < 0) {
							endPos = lastValues.Length;
							cutPos = lastValues.Length;
						}

						string currentValue = lastValues.Substring(0, endPos);
						lastValues = lastValues.Substring(cutPos, lastValues.Length - cutPos);

						int valueInt = 0;
						valueInt = Convert.ToInt32(currentValue);

						if (valueInt > 12) { //Sensitivity goes here.
							PushButton(i);
						} else {
							ReleaseButton(i);
						}
					}
				}
			} catch { }
		}

		private void TextBox_TextChanged(object sender, TextChangedEventArgs e)
		{

		}

		private void SerialButton_Click(object sender, RoutedEventArgs e)
		{
			Properties.Settings.Default.COM_Port = SerialBox.Text;
			Properties.Settings.Default.Save();
			ArduinoInit();
		}
	}
}
